#!/usr/bin/perl -w

# LSB Test Execution Framework
# Configuration Save Module (tests_conf_save.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use Fcntl ':flock';
use Fcntl ':seek';
use Cwd 'cwd';

autoflush_on();

# Adds directory contents to the list (recursively)
sub add_dir_to_list($$) {
	my ($dir, $file_list) = @_;
	if (opendir(DIR, $dir)) {
		foreach (sort(grep(!/^\.+$/, readdir(DIR)))) {
			if (-f "$dir/$_") {
				$file_list->{"$dir/$_"} = 1;
			}
			else {
				&add_dir_to_list("$dir/$_", $file_list);
			}
		}
		closedir(DIR);
	}
}

# Creates the INFO file for a particular journal and fills it with the information needed
sub get_journal_suffix_anchor($) {
	my ($fpath) = @_;
	my $fname = $fpath;
	$fname =~ s|^.*/([^/]*)$|$1|;
	my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($fpath);
	return ("$fname.$dev-$ino.journal", "${fname}_${dev}_${ino}");
}

sub create_app_journal_info($$$$$) {
	my ($jpath, $fpath, $path_root_real, $path_root_show, $no_check) = @_;
	my $ftype = `file $fpath`;
	if ($no_check or ($ftype =~ m/executable/) or ($ftype =~ m/shared object/) or ($fpath =~ m/\.(pl|py)$/)) {
		my ($suffix, $anchor) = get_journal_suffix_anchor($fpath);
		if (open(INFO_FILE, ">$jpath/lsbappchk.$suffix.INFO")) {
			$fpath =~ s/^$path_root_real/$path_root_show\|/;
			print INFO_FILE "TITLE: $fpath\nANCHOR: lsbappchk_$anchor\nJOURNAL_FILENAME: lsbappchk.$suffix\n";
			close(INFO_FILE);
		}
	}
}

sub create_pkg_journal_info($$) {
	my ($jpath, $fpath) = @_;
	my ($suffix, $anchor) = get_journal_suffix_anchor($fpath);
	if (open(INFO_FILE, ">$jpath/lsbpkgchk.$suffix.INFO")) {
		$fpath =~ s/$/\|/;
		print INFO_FILE "TITLE: $fpath\nANCHOR: lsbpkgchk_$anchor\nJOURNAL_FILENAME: lsbpkgchk.$suffix\n";
		close(INFO_FILE);
	}
	return "$jpath/lsbpkgchk.$suffix";
}

my $conf_page = "tests_conf.pl";
if ($_GET{"goto"}) {
	$conf_page = $_GET{"goto"};
	$conf_page =~ s/%23/#/;
	$_POST{"form_action"} = "";
}
my $session_profile = ($_COOKIE{"session_id"} ? "~session.".$SERVER_PARAM{"PEER_IP"}.".".$_COOKIE{"session_id"} : "");

my $error;
my $new_url = "";
my $save_profile_name = "";

if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
	system("mkdir -p ".$SERVER_PARAM{"APP_DATA"}."/profiles");
	if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
		$error = "";
	}
}

if (!defined($error)) {
	if ((!$_POST{"form_action"} or ($_POST{"form_action"} eq "")) and (!$_GET{"goto"})) {
		$error = "Script action not specified!";
	}
	elsif ($_POST{"form_action"} eq "load_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		elsif (!-f $SERVER_PARAM{"APP_DATA"}."/profiles/".$_POST{"edit_profile_name"}) {
			$error = "File not found:<br />".$SERVER_PARAM{"APP_DATA"}."/profiles/".$_POST{"edit_profile_name"};
		}
		else {
			system("cp \"".$SERVER_PARAM{"APP_DATA"}."/profiles/".$_POST{"edit_profile_name"}."\" \"".$SERVER_PARAM{"APP_DATA"}."/profiles/$session_profile\" > /dev/null");
			$new_url = $conf_page;
		}
	}
	elsif ($_POST{"form_action"} eq "run_tests") {
		if ($session_profile eq "") {
			$error = "Cannot retrieve session ID! Please, allow cookies in your browser.";
		}
		else {
			$save_profile_name = $session_profile;
			$new_url = "tests_results.pl";
		}
	}
	elsif ($_POST{"form_action"} eq "save_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		else {
			$save_profile_name = $_POST{"edit_profile_name"};
			$new_url = $conf_page;
		}
	}
	elsif ($_POST{"form_action"} eq "delete_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		elsif (!-f $SERVER_PARAM{"APP_DATA"}."/profiles/".$_POST{"edit_profile_name"}) {
			$error = "File not found:<br />".$SERVER_PARAM{"APP_DATA"}."/profiles/".$_POST{"edit_profile_name"};
		}
		else {
			unlink($SERVER_PARAM{"APP_DATA"}."/profiles/".$_POST{"edit_profile_name"});
			$save_profile_name = $session_profile;
			$new_url = $conf_page;
		}
	}
	elsif ($_GET{"goto"}) {
		$save_profile_name = $session_profile;
		$new_url = $conf_page;
	}
}

if (!defined($error)) {
	if ($save_profile_name ne "") {
		my $full_save_name = $SERVER_PARAM{"APP_DATA"}."/profiles/$save_profile_name";
		if (open(FILE, "> $full_save_name")) {
			my $architecture = ($_POST{"architecture"} or guess_platform(`uname -m`));
			my $lsb_version  = ($_POST{"lsb_version"} or "3.2");
			my $lsb_profile  = ($_POST{"lsb_profile"} or "desktop");
			print FILE "APPLICATION:     ".$_POST{"application"}."\n" if ($_POST{"application"});
			if ($_POST{"paths"}) {
				$_POST{"paths"} =~ s/\r\n/\n/g;
				my @paths_lines = split(/\n/, $_POST{"paths"});
				foreach (@paths_lines) {
					print FILE "PATH:            $_\n";
				}
			}
			if ($_POST{"packages"}) {
				$_POST{"packages"} =~ s/\r\n/\n/g;
				my @packages_lines = split(/\n/, $_POST{"packages"});
				foreach (@packages_lines) {
					print FILE "PACKAGE:         $_\n";
				}
			}
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "LSB_VERSION:     $lsb_version\n";
			print FILE "LSB_PROFILE:     $lsb_profile\n";

			if ($_POST{"comments"}) {
				$_POST{"comments"} =~ s/\r\n/\n/g;
				$_POST{"comments"} =~ s/&/&amp;/g;
				$_POST{"comments"} =~ s/</&lt;/g;
				$_POST{"comments"} =~ s/>/&gt;/g;
				my @comments_lines = split(/\n/, $_POST{"comments"});
				foreach (@comments_lines) {
					print FILE "COMMENTS: $_\n";
				}
			}

			close(FILE);

			if ($_POST{"form_action"} eq "save_profile") {
				system("cp \"$full_save_name\" \"".$SERVER_PARAM{"APP_DATA"}."/profiles/$session_profile\" > /dev/null");
			}
			elsif ($_POST{"form_action"} eq "run_tests") {
				if (!$_POST{"application"}) {
					$error = "Application name is not specified.";
				}
				else {
					print "HTTP/1.0 200 OK\n";
					print "Content-type: text/html\n\n";
					print_header("Test Execution", "conf");
					
					# Show timer with waiting message
					print <<DATA;
<div id="progress_info">
<font color="green"><b>Please, wait while the test is running...</b></font><br />
Time elapsed: <span id="timer" style="font-family: monospace; color: blue;">00:00</span></p>
</div>

<script language="javascript" type="text/javascript">
// <![CDATA[
var elapsed_time = 0;
var finished = 0;
function update_timer() {
	++elapsed_time;
	var elem = document.getElementById("timer");
	if (elem && !finished) {
		var m = Math.floor(elapsed_time / 60);
		var s = elapsed_time - m * 60;
		var tmp = "";
		tmp += (m < 10) ? "0" + m : m;
		tmp += ":";
		tmp += (s < 10) ? "0" + s : s;
		elem.innerHTML = tmp;
		setTimeout("update_timer()", 1000);
	}
}

setTimeout("update_timer()", 1000);
// ]]>
</script>
DATA
					print_footer(1);

					my $app_name = $_POST{"application"};
					my $app_name_esc = $app_name;
					$app_name_esc =~ s/ /\\ /g;

					# Create results directory
					my @clean_dirs = ();
					my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
					my $host_name = `uname -n`;
					$host_name =~ s/[\r\n]//g;
					my $results_dir = sprintf("$architecture-$host_name-$app_name-%04d-%02d-%02d-%02dh-%02dm-%02ds", ($year + 1900), ($mon + 1), $mday, $hour, $min, $sec);
					my $results_dir_esc = $results_dir;
					$results_dir_esc =~ s/ /\\ /g;
					system("mkdir -p ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc/test_journals");
					system("mkdir -p ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc/test_logs");
					if ((!-d $CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals") or (!-d $CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs")) {
						$error = "Could not create subdirectories for storing the test results in<br />$results_dir<br />$!";
					}
					else {
						# Prepare data for testing
						my $tests_output = $CONFIG{"RESULTS_DIR"}."/$results_dir_esc/test_logs/tests_output";
						my $cmd_line = $CONFIG{"TESTS_DIR"}."/lsbappchk -r $lsb_version -j -o ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc/test_journals -l /tmp/$app_name_esc.list.$$ ";
						if ($lsb_profile eq "desktop") {
							$cmd_line .= "-Tcore,c++,desktop";
						}
						elsif ($lsb_profile eq "core") {
							$cmd_line .= "-Tcore,c++";
						}
						$idx = 0;
						my %file_list;
						my @failed_files = ();
						my @rpm_files = ();
						my @list_appchk = ();
						my @list_db = ();
						if ($_POST{"paths"}) {
							my @paths_lines = split(/\n/, $_POST{"paths"});
							foreach my $path (@paths_lines) {
								if (-d $path) {
									%file_list = ();
									add_dir_to_list($path, \%file_list);
									foreach (sort keys %file_list) {
										create_app_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $_, $path, $path, 0);
										push @list_appchk, $_;
										push @list_db, $_;
									}
								}
								elsif ($path =~ m/\.rpm$/) {
									system("echo \"Unpacking RPM package $path\" >>$tests_output");
									$idx++;
									mkdir("/tmp/pkg-unpack.$$-$idx");
									push @clean_dirs, "/tmp/pkg-unpack.$$-$idx";
									my $path_esc = $path;
									$path_esc =~ s/ /\\ /g;
									system("cd /tmp/pkg-unpack.$$-$idx; rpm2cpio $path_esc 2>>$tests_output | cpio -id 2>>$tests_output");
									if (!$?) {
										my $j_path = create_pkg_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $path);
										push @rpm_files, $path_esc;
										push @rpm_journals, $j_path;
										push @list_db, $path;
										%file_list = ();
										add_dir_to_list("/tmp/pkg-unpack.$$-$idx", \%file_list);
										foreach (sort keys %file_list) {
											create_app_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $_, "/tmp/pkg-unpack.$$-$idx", $path, 0);
											push @list_appchk, $_;
										}
									}
									else {
										push @failed_files, $path;
									}
								}
								elsif (($path =~ m/\.tgz$/) or ($path =~ m/\.tar\.gz$/)) {
									system("echo \"Unpacking TGZ package $path\" >>$tests_output");
									$idx++;
									mkdir("/tmp/pkg-unpack.$$-$idx");
									push @clean_dirs, "/tmp/pkg-unpack.$$-$idx";
									my $path_esc = $path;
									$path_esc =~ s/ /\\ /g;
									system("cd /tmp/pkg-unpack.$$-$idx; tar xzf $path_esc 2>>$tests_output");
									if (!$?) {
										push @list_db, $path;
										%file_list = ();
										add_dir_to_list("/tmp/pkg-unpack.$$-$idx", \%file_list);
										foreach (sort keys %file_list) {
											create_app_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $_, "/tmp/pkg-unpack.$$-$idx", $path, 0);
											push @list_appchk, $_;
										}
									}
									else {
										push @failed_files, $path;
									}
								}
								else {
									create_app_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $path, "", "", 1);
									push @list_appchk, $path;
									push @list_db, $path;
								}
							}
						}
						if ($_POST{"packages"}) {
							my @packages_lines = split(/\n/, $_POST{"packages"});
							foreach my $pkg (@packages_lines) {
								my $pkg_list = ($is_packager_rpm ? `rpm -ql $pkg` : `dpkg -L $pkg`);
								foreach (split(/\n/, $pkg_list)) {
									if (!-d) {
										create_app_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $_, "", "PKG: $pkg", 0);
										push @list_appchk, $_;
										push @list_db, $_;
									}
								}
							}
						}
						my @perl_files = ();
						my @python_files = ();
						if (open(LIST_FILE, ">/tmp/$app_name.list.$$")) {
							foreach (@list_appchk) {
								if (m/\.pl$/ or m/\.pm$/) {
									push @perl_files, $_;
								}
								elsif (m/\.py$/) {
									push @python_files, $_;
								}
								else {
									print LIST_FILE "$_\n";
								}
							}
							close(LIST_FILE);
						}
						if (open(LIST_FILE_DB, ">/tmp/$app_name.list_db.$$")) {
							foreach (@list_db) {
								print LIST_FILE_DB "$_\n";
							}
							close(LIST_FILE_DB);
						}
						if (scalar(@failed_files) == 0) {
							for (my $i = 0; $i < scalar(@rpm_files); $i++) {
								my $cmd_line_pkg = $CONFIG{"TESTS_DIR"}."/lsbpkgchk -r $lsb_version -j ".$rpm_journals[$i];
								if ($lsb_profile eq "desktop") {
									$cmd_line_pkg .= " -Tcore,c++,desktop ";
								}
								elsif ($lsb_profile eq "core") {
									$cmd_line_pkg .= " -Tcore,c++ ";
								}
								$cmd_line_pkg .= $rpm_files[$i];
								system("$cmd_line_pkg >>$tests_output 2>&1");
							}
							mkdir("/tmp/atk-journals.$$");
							my $old_dir = cwd();
							chdir("/tmp/atk-journals.$$");
							for (my $i = 0; $i < scalar(@perl_files); $i++) {
								my $cmd_line_pl = $CONFIG{"TESTS_DIR"}."/lsbappchk.pl -j ".$perl_files[$i];
								system("$cmd_line_pl >>$tests_output 2>&1");
								my ($suffix, $anchor) = get_journal_suffix_anchor($perl_files[$i]);
								my $fname = $perl_files[$i];
								$fname =~ s|^.*/([^/]*)$|$1|;
								rename("/tmp/atk-journals.$$/journal.lsbappchk.pl.$fname", $CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals/lsbappchk.$suffix");
							}
							for (my $i = 0; $i < scalar(@python_files); $i++) {
								my $cmd_line_py = $CONFIG{"TESTS_DIR"}."/lsbappchk.py -j ".$python_files[$i];
								system("$cmd_line_py >>$tests_output 2>&1");
								my ($suffix, $anchor) = get_journal_suffix_anchor($python_files[$i]);
								my $fname = $python_files[$i];
								$fname =~ s|^.*/([^/]*)$|$1|;
								rename("/tmp/atk-journals.$$/journal.lsbappchk.py.$fname", $CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals/lsbappchk.$suffix");
							}
							chdir($old_dir);
							rmdir("/tmp/atk-journals.$$");
							system("$cmd_line >>$tests_output 2>&1");
							system("touch ".$CONFIG{"RESULTS_DIR"}."/HISTORY");
							if (open(FILE, "+<".$CONFIG{"RESULTS_DIR"}."/HISTORY")) {
								flock(FILE, LOCK_EX);
								seek(FILE, 0, SEEK_END);
								print FILE "$results_dir\n";
								flock(FILE, LOCK_UN);
								close(FILE);
							}
							else {
								system("echo \"Could not open HISTORY file!\nThe test results will not be listed on the Results page.\" >>$tests_output 2>&1");
							}
							if (open(COMMENTS_FILE, ">".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/test_comments")) {
								print COMMENTS_FILE $_POST{"comments"};
								close(COMMENTS_FILE);
							}
							system("echo \"$lsb_version\" >> ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc/test_logs/lsb_version");
							system("echo \"$architecture\" >> ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc/test_logs/host_platform");

							# Now all preparations are done, generate the report
							if (-s $CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/tests_output" != 0) {
								system("echo \"\" >>$tests_output");
							}
							system($CONFIG{"UTILS_DIR"}."/report.pl ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc -n $app_name_esc -o lsbappchk-$results_dir_esc.tgz >>$tests_output 2>&1");

							# Generate application analysis data
							# apptodb needs PM file located in its directory.
							chdir($CONFIG{"UTILS_DIR"});
							system("echo \"\" >>$tests_output");
							system("echo \"Collecting data about application files\" >>$tests_output");
							system($CONFIG{"UTILS_DIR"}."/apptodb.pl -a $architecture -l /tmp/$app_name_esc.list_db.$$ -o ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc/${app_name_esc}_${architecture}_upload_data $app_name_esc >>$tests_output 2>&1");
							$new_url = "tests_results.pl?details=$results_dir";
						}
						else {
							$error = "Could not unpack the following files:<br />".join("<br />", @failed_files);
						}
						unlink("/tmp/$app_name.list.$$");
						unlink("/tmp/$app_name.list_db.$$");
						foreach (@clean_dirs) {
							system("rm -rf $_ >/dev/null 2>&1");
						}
					}
					if (defined($error)) {
						system("rm -rf ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc >/dev/null 2>&1");
						escape($error);
						print <<DATA;
<script language="javascript" type="text/javascript">
// <![CDATA[
finished = 1;
document.cookie="last_error_text=$error; path=/";
location.replace("$conf_page");
// ]]>
</script>
DATA
					}
					else {
						print <<DATA;
<script language="javascript" type="text/javascript">
// <![CDATA[
finished = 1;
location.replace("$new_url");
// ]]>
</script>
DATA
					}
					print "</body>\n</html>\n";
					exit;
				}
			}
		}
		else {
			$error = "Cannot open file $save_profile_name for writing!<br />$!";
		}
	}
}

if (defined($error)) {
	escape($error);
	print "HTTP/1.0 302 Moved\n";
	print "Set-Cookie: last_error_text=$error; path=/;\n";
	print "Location: $conf_page\n\n";
}
else {
	print "HTTP/1.0 302 Moved\n";
	print "Location: $new_url\n\n";
}
